#!/usr/bin/env python
#EagleEye
#http://launchpad.net/eagleeye
#(c) Clement Lorteau <northern_lights@users.sourceforge.net>

version = "0.4"

import sys
try:
 	import pygtk
	pygtk.require("2.0")
except:
	pass
try:
	import gtk
	import gconf
	import gnomekeyring
	from optparse import OptionParser
	import gobject
	gobject.threads_init()
except:
	sys.exit(1)
try:
	import LaunchpadIntegration
	launchpad = True
except:
	launchpad = False

backends = []
try:
	import gtkvnc
	backends.append('gtkvnc')
except:
	pass
try:
	from tacix.client.TXClient import TXClient
	backends.append('tacix')
except:
	pass

import os
from xdg.BaseDirectory import *

#internat'
import locale
import gettext
APP = 'eagleeye'
DIR = 'locale'
locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gettext.install(APP, DIR, unicode=1)
_ = gettext.gettext

def wait():
	while gtk.events_pending():
		gtk.main_iteration()

def get_xdg_data_file(filename):
	for path in load_data_paths(APP, filename):
		return path

class GtkVncViewer:

	def __init__(self):
		self.parse_options()	
		#build GUI
		self.wTree = gtk.Builder()
		file = get_xdg_data_file("eagleeye.ui")
		self.wTree.add_from_file(file)
		self.dialog = self.wTree.get_object("connectDialog")
		self.about = self.wTree.get_object("aboutDialog")
		self.about.set_version(version)
		self.sessionNameDialog = self.wTree.get_object("sessionNameDialog")
		self.iconview = self.wTree.get_object("iconview1")

		for backend in backends:
			self.wTree.get_object("typeStore").append([backend])

		""" session_name, pix, server_address, username, profile """
		self.model = gtk.ListStore (str,gtk.gdk.Pixbuf,str,str,str)
		self.iconview.set_model(self.model)
		self.iconview.set_text_column(0)
		self.iconview.set_pixbuf_column(1)
		self.helpMenu = self.wTree.get_object("helpMenu")
		self.helpButton = self.wTree.get_object("helpButton")
		self.helpMenu.attach_to_widget(self.helpButton, None)
		if (launchpad):
			LaunchpadIntegration.set_sourcepackagename("eagleeye")
			LaunchpadIntegration.add_items(self.helpMenu,0,False,False)
		else:
			self.helpButton.set_sensitive(False)

		self.iconview.connect('button-press-event', self.show_iconview_context_menu)
		self.wTree.get_object("preferencesButton").connect('clicked', self.show_preferences_dialog)
		#self.wTree.get_object("typeEntry").connect('changed', self.set_selected_type)

		if (self.dialog):
			#Create our dictionay and connect it
			dic = { "on_quitButton_clicked" : gtk.main_quit,
				"on_aboutButton_clicked" : self.show_about_dialog,
				"on_aboutDialog_response" : self.handle_about_dialog_answer,
				"on_aboutDialog_delete_event" : self.hide_about_dialog,
				"on_connectButton_clicked" : self.connect,
				"on_addButton_clicked" : self.add_server,
				"on_cancelAddButton_clicked" : self.hideSessionNameDialog,
				"on_saveSessionButton_clicked" : self.save_server,
				"on_iconview1_selection_changed" : self.selected,
				"on_iconview1_item_activated" : self.activated,
				"on_delButton_clicked" : self.delete_clicked,
				"on_helpButton_clicked" : self.helpMenuPop,
				}
			self.wTree.connect_signals(dic)
			self.dialog.show()
		
		#read stored credentials
		GCONF_AUTH_KEY = "/apps/eagleeye/sessions"
		gconfclient = gconf.client_get_default()
		gconfclient.add_dir (GCONF_AUTH_KEY,
			gconf.CLIENT_PRELOAD_RECURSIVE)
		dirs = gconfclient.all_dirs(GCONF_AUTH_KEY)
		pixbuf = self.iconview.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)
		self.model.append(['Custom', pixbuf, '', '', ''])
		if len(dirs) == 0:
			return #found nothing
		for i in range(len(dirs)):
			session_name_value = gconfclient.get_entry(dirs[i]+"/session_name","", False).get_value()
			if session_name_value is None:
				continue
			session_name = session_name_value.get_string()
			if session_name is None or session_name == "":
				continue
			props = {}
			for prop in ('server_address', 'session_name', 'username', 'profile'):
				value = gconfclient.get_entry(dirs[i]+"/"+prop,"", False).get_value()
				if value is not None:
					props[prop] = value.get_string()
				else:
					props[prop] = ""
			self.model.append([props['session_name'], pixbuf, props['server_address'], props['username'], props['profile']])

		#if a server was specified at startup, connect to it
		if self.startup_options.server:
			res = self.find_server(self.startup_options.server)
			if (res): #found, connect
				iter = res[3]
				self.iconview.select_path(iter)
				wait()
				self.connect(None)
			else: #not found, just fill the server name in the dialog
				server_textbox = self.wTree.get_object("serverEntry")
				user_textbox = self.wTree.get_object("usernameEntry")
				server_textbox.set_text(self.startup_options.server)
				user_textbox.grab_focus()
	
	def helpMenuPop (self, data):
		self.helpMenu.popup(None, None, None, 0, 0, gtk.get_current_event_time())

	def delete_clicked (self, data):		
		select = self.iconview.get_selected_items()
		if len(select) == 0:
			print _("nothing to delete")
			return

		i = select[0][0]
		iter = self.model.get_iter(i)
		s = self.model.get(iter, 0, 2, 3, 4)
		session = s[0]
		
		#ask confirmation
		dialog = gtk.MessageDialog (self.dialog,
			gtk.MESSAGE_QUESTION,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.BUTTONS_YES_NO,
			_("Are you sure you want remove the session \"%s\"?") % session)
		r = dialog.run()
		dialog.destroy()
		if (r == gtk.RESPONSE_NO):
			print _("deletion canceled")
		else:
			GCONF_AUTH_KEY = "/apps/eagleeye/sessions/" + session
			gconfclient = gconf.client_get_default()
			gconfclient.add_dir (GCONF_AUTH_KEY,
				gconf.CLIENT_PRELOAD_RECURSIVE)
			auth_token_value = gconfclient.get_entry(GCONF_AUTH_KEY+"/keyring_token","", False).get_value()
			if auth_token_value is not None:
				auth_token = auth_token_value.get_int()
			if auth_token_value is not None and auth_token > 0:
				try:
					#delete gnome-keyring entry
					keyring = gnomekeyring.get_default_keyring_sync()
					gnomekeyring.item_delete_sync(keyring, auth_token)
					print session+" deleted"
				except gnomekeyring.DeniedError:
					auth_token = 0
					print _("gnome-keyring access denied, could not delete")
			else:
				print _("keyring password not deleted")
			#delete gconf entry
			gconfclient.recursive_unset(GCONF_AUTH_KEY, gconf.UNSET_INCLUDING_SCHEMA_NAMES)
			gconfclient.remove_dir(GCONF_AUTH_KEY)
			#remove from icon view
			self.model.remove(iter)

	def activated (self, widget, data):
		self.connect(widget)

	def selected (self, data):
		select = self.iconview.get_selected_items()
		if len(select) == 0:
			return
		i = select[0][0]
		iter = self.model.get_iter(i)
		s = self.model.get(iter, 0, 2, 3, 4)
		self.wTree.get_object("serverEntry").set_text(s[1])
		if s[0] == "Custom":
			self.wTree.get_object("delButton").set_sensitive(False)
			self.wTree.get_object("preferencesButton").set_sensitive(False)
			self.wTree.get_object("sessionNameEntry").set_text("")
		else:
			self.wTree.get_object("delButton").set_sensitive(True)
			self.wTree.get_object("preferencesButton").set_sensitive(True)
			self.wTree.get_object("sessionNameEntry").set_text(s[0])
		self.wTree.get_object("usernameEntry").set_text(s[2])
		self.wTree.get_object("pwdEntry").set_text("")
		if s[3] != '':
			self.wTree.get_object("typeEntry").set_active(backends.index(s[3]))

	def get_backend (self):
		backend_name = self.wTree.get_object("typeEntry").get_active_text()
		if backend_name == 'tacix':
			return TaciXBackend(self)
		elif backend_name == 'gtkvnc':
			return GtkVncBackend(self)


	#finds the server 'name', and return its credentials and its iconview path
	#if not found, return False
	def find_server (self, name):
		iter = self.model.get_iter_first()
		while (iter != None):
			row = self.model.get(iter, 0) #server
			if row[0] == name:
				row += ( self.model.get_path(iter), )
				return row
			iter = self.model.iter_next(iter)
			return False
	
	def hideSessionNameDialog (self, data):
		self.sessionNameDialog.hide()
		select = self.iconview.get_selected_items()
		if len(select) == 0:
			self.wTree.get_object("sessionNameEntry").set_text("")
			return
		i = select[0][0]
		iter = self.model.get_iter(i)
		s = self.model.get(iter,0, 2, 3)
		if s[0] == "Custom":
			self.wTree.get_object("sessionNameEntry").set_text("")
		else:
			self.wTree.get_object("sessionNameEntry").set_text(s[1])

	def add_server (self, data):
		server = self.wTree.get_object("serverEntry").get_text()
		if server == "":
			return
		sessionName = self.wTree.get_object("sessionNameEntry").get_text()
		if sessionName == "":
			sessionName = server
		self.wTree.get_object("sessionNameEntry").set_text(sessionName)
		self.sessionNameDialog.show()

	def save_server (self, data):
		self.sessionNameDialog.hide()
		#add it to the iconview
		pixbuf = self.iconview.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)
		username = self.wTree.get_object("usernameEntry").get_text()
		password = self.wTree.get_object("pwdEntry").get_text()
		server = self.wTree.get_object("serverEntry").get_text()
		session = self.wTree.get_object("sessionNameEntry").get_text()
		save_password = self.wTree.get_object("savePasswordCheckbox").get_active()
		profile = self.wTree.get_object("typeEntry").get_active_text()
		if session == "":
			session = server
		self.model.append([session, pixbuf, server, username, profile])

		#now save it in gnome-keyring
		GCONF_AUTH_KEY = "/apps/eagleeye/sessions/"+session
		gconf.client_get_default().set_string(GCONF_AUTH_KEY+'/profile', profile)
		gconf.client_get_default().set_string(GCONF_AUTH_KEY+'/session_name', session)
		gconf.client_get_default().set_string(GCONF_AUTH_KEY+'/server_address', server)
		gconf.client_get_default().set_string(GCONF_AUTH_KEY+'/username', username)
		gconf.client_get_default().set_bool(GCONF_AUTH_KEY+'/save_password', save_password)

		if save_password:
			keyring = gnomekeyring.get_default_keyring_sync()
			auth_token = gnomekeyring.item_create_sync(
				keyring,
				gnomekeyring.ITEM_GENERIC_SECRET,
				"Password for "+session+" remote desktop session",
				dict(appname="eagleeye, credentials of remote desktop session \""+session+"\""),
				password, True)
			gconf.client_get_default().set_int(GCONF_AUTH_KEY+'/keyring_token', auth_token)
		else:
			gconf.client_get_default().set_int(GCONF_AUTH_KEY+'/keyring_token', 0)

	def handle_about_dialog_answer(self, widget, data):
		if(data==-6):
			self.about.hide()

	def show_about_dialog(self, widget):
		self.about.show()

	def hide_about_dialog(self, widget, data):
		self.about.hide()
		return True

	def show_iconview_context_menu(self, widget, event):
		if event.button == 3:
			self.wTree.get_object("iconviewContextMenu").popup(None, None, None, event.button, 0, gtk.get_current_event_time())

	def get_gconf_key(self):
		session_name = self.wTree.get_object("sessionNameEntry").get_text()
		if session_name == '':
			return None
		GCONF_AUTH_KEY = "/apps/eagleeye/sessions/" + session_name
		gconfclient = gconf.client_get_default()
		entry = gconfclient.get_entry(GCONF_AUTH_KEY+"/server_address",
			"", False)
		server = entry.get_value().get_string()

		if server == self.wTree.get_object("serverEntry").get_text():
			return GCONF_AUTH_KEY
		return None

	def show_preferences_dialog(self, widget):
		self.get_backend().show_preferences_dialog()

	def connect(self, window):
		self.dialog.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
		username = self.wTree.get_object("usernameEntry").get_text()
		password = self.wTree.get_object("pwdEntry").get_text()
		GCONF_AUTH_KEY = self.get_gconf_key()
		if password == '' and GCONF_AUTH_KEY is not None:
			# Get the password
			gconfclient = gconf.client_get_default()
			gconfclient.add_dir (GCONF_AUTH_KEY,
				gconf.CLIENT_PRELOAD_RECURSIVE)
			entry = gconfclient.get_entry(GCONF_AUTH_KEY+"/keyring_token","", False)
			auth_token = entry.get_value().get_int()

			password = ""
			if auth_token > 0:
				try:
					keyring = gnomekeyring.get_default_keyring_sync()
					password = gnomekeyring.item_get_info_sync(keyring, auth_token).get_secret()
				except gnomekeyring.DeniedError:
					auth_token = 0
					print _("gnome-keyring access denied")


		server = self.wTree.get_object("serverEntry").get_text()
		print _("Connecting to %s...") % server 
		serverport = server.split(':')
		server = serverport[0]
		try:
			port = serverport[1]
		except:
			port = None
		self.get_backend().connect(server, port, username, password)
		self.dialog.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.ARROW))

	def parse_options(self):
		parser = OptionParser()
		parser.add_option("-s", "--server", dest="server",
				action="store", type="string",
				help=_("if server is known by eagleeye, immediately connect to it"),
				metavar=_("server"))
		(options, args) = parser.parse_args()
		self.startup_options = options


class TaciXBackend:
	def __init__(self, eagleeye_dialog):
		self.dialog = eagleeye_dialog.dialog
		self.txclient = None
		self.eagleeye_dialog = eagleeye_dialog

	def connect(self, server, port, username, password):
		if port is None:
			port = '22'
			port = '22'
		message = None
		try:
			txclient = TXClient(host=server, port=int(port))
			if txclient.connect(username, password):
				print _("Connected")
				self.connect_dialog = gtk.MessageDialog (self.dialog,
					gtk.DIALOG_DESTROY_WITH_PARENT,
					gtk.MESSAGE_INFO,
					gtk.BUTTONS_NONE,
					_("Connecting to host"))
				self.connect_dialog.show()
				self.dialog.hide()
				txclient.run(disconnect_callback=self.disconnected, started_signal_callback=self.started, update_status_callback=self.update_connect_dialog)
				self.txclient = txclient
				return
		except Exception, e:
			message = repr(e)

		if message is None:
			message = _("Could not connect to the server")

		dialog = gtk.MessageDialog (self.dialog,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_INFO,
			gtk.BUTTONS_OK,
			message)
		dialog.run()
		dialog.destroy()

	def started(self):
		self.connect_dialog.destroy()

	def update_connect_dialog(self, status):
		self.connect_dialog.set_markup('Status is %s' % status)

	def disconnected(self):
		print _("Disconnected")
		print _("Bye.")
		if (self.dialog):
			self.dialog.destroy()
		# Hack to make the program exit
		#gtk.main_quit()
		sys.exit(0)

	def show_preferences_dialog(self):
		select = self.eagleeye_dialog.iconview.get_selected_items()
		if len(select) == 0:
			return
		i = select[0][0]
		iter = self.eagleeye_dialog.model.get_iter(i)
		s = self.eagleeye_dialog.model.get(iter,0, 2, 3)
		if s[0] == 'Custom':
			return

		txclient = TXClient(session_name=s[0])
		from tacix.gtk.GTKClient import GTKClient
		gtk_client = GTKClient(txclient)
		gtk_client.show_preferences()


class GtkVncBackend:
	def __init__(self, eagleeye_dialog):
		self.vnc=gtkvnc.Display()

		self.wTree = gtk.Builder()
		self.wTree.add_from_file(get_xdg_data_file("vnc.ui"))
		self.dialog = eagleeye_dialog.dialog

		self.window = self.wTree.get_object("window")
		self.window_label = self.wTree.get_object("window_label")
		self.window_toolbar_note = self.wTree.get_object("toolbar_note")
		self.window_toolbar = self.wTree.get_object("toolbar")
		self.layout = self.wTree.get_object("viewport1")
		self.scrolledwindow = self.wTree.get_object("scrolledwindow1")
		self.fullscreenButton = self.wTree.get_object("fullscreenButton")
		self.fullscreenButton.set_active(False)
		self.keysButton = self.wTree.get_object("keysButton")
		self.keysMenu = self.wTree.get_object("keysMenu")
		self.keysMenu.attach_to_widget(self.keysButton, None)

		self.window.connect("destroy", gtk.main_quit)

		if (self.dialog):
			#Create our dictionay and connect it
			dic = { 
				"on_disconnectButton_clicked" : self.disconnect,
				"on_screenshotButton_clicked" : self.screenshot,
				"on_window_delete_event" : self.close_window,
				"on_togglebutton1_toggled" : self.fullscreen,
				"on_toolbar_note_entered" : self.show_hide_toolbar,
				"on_window_motion_notify_event" : self.mouse_moved_in_window,
				"on_desktopIconButton_clicked" : self.icon_on_desktop,
				"on_CtrlAltDelmenuitem_activate": self.send_cad,
				"on_CtrlAltBackmenuitem_activate": self.send_cab,
				"on_CtrlEscmenuitem_activate": self.send_ce,
				"on_keysButton_clicked" : self.keysMenuPop
				}
			self.wTree.connect_signals(dic)


	def keysMenuPop (self, data):
		self.keysMenu.popup(None, None, None, 0, 0, gtk.get_current_event_time())
	
	def screenshot (self, data):
		homeDir = os.environ.get('HOME', None) #=> can't work on Windows
		pix = self.vnc.get_pixbuf()
		pix.save(homeDir+"/vnc.png", "png", { "tEXt::Generator App": "eagleeye" })
		dialog = gtk.MessageDialog (self.window,
			gtk.MESSAGE_INFO,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.BUTTONS_OK,
			_("Screenshot saved in")+" "+homeDir+"/vnc.png")
		dialog.run()
		dialog.destroy()
		return False
	
	def fullscreen (self, data):
		if (self.fullscreenButton.get_active()):
			self.window.fullscreen()
			self.window_toolbar_note.show_all()
			self.window_toolbar.hide_all()
		else:
			self.window.unfullscreen()
			self.window_toolbar_note.hide_all()
			self.window_toolbar.show_all()
		return False

	def send_cad (self, data):
		self.vnc.send_keys(["Control_L", "Alt_L", "Delete"])
		print _("Sent Ctrl+Alt+Delete")
		self.vnc.grab_focus()

	def send_cab (self, data):
		self.vnc.send_keys(["Control_L", "Alt_L", "BackSpace"])
		print _("Sent Ctrl+Alt+BackSpace")
		self.vnc.grab_focus()

	def send_ce (self, data):
		self.vnc.send_keys(["Control_L", "Escape"])
		print _("Sent Ctrl+Escape")
		self.vnc.grab_focus()

	def quit():
		self.vnc.close()
		print _("Bye.")
		if (self.window):
			self.window.destroy()

	def close_window(self, widget, data):
		quit()
		return False
	
	def disconnect(self, data):
		quit()

	def show_hide_toolbar(self, widget, data):
		print "show_hide_toolbar"
		return False

	#if in fullscreen and mouse on top, show toolbar
	def mouse_moved_in_window(self, widget, data):
		coords = self.window.window.get_pointer()
		y = coords[1]
		if y <= 5 and self.fullscreenButton.get_active():
			self.window_toolbar.show_all()
			#setup timer that will hide toolbar when ended
			gobject.timeout_add(2000, self.window_toolbar.hide_all) #2 sec.

	def icon_on_desktop(self, data):
		server = self.current_server
		comment = _("Connect to the remote desktop: %s" % (server))
		text = """
[Desktop Entry]
Name=%s
Comment=%s
Exec=eagleeye -s %s
Icon=/usr/share/eagleeye/data/eagleeye_64.png
StartupNotify=true
Terminal=false
Type=Application
Categories=Network;
""" % ("VNC: "+server, comment, server)
		open(os.path.join(os.environ['HOME']+"/Desktop",
				server+".desktop"),"w").write(text)

	def connect(self, server, port, username, password):
		self.vnc.set_credential(gtkvnc.CREDENTIAL_USERNAME, username)
		self.vnc.set_credential(gtkvnc.CREDENTIAL_PASSWORD, password)
		self.vnc.set_credential(gtkvnc.CREDENTIAL_CLIENTNAME, "eagleeye")
		if port is None:
			port = "5900"
		self.vnc.open_host(server, port)
		#vnc.connect("vnc-auth-credential", self.vnc_auth_cred)
		self.vnc.connect("vnc-connected", self.vnc_connected, self)
		self.vnc.connect("vnc-initialized", self.vnc_initialized, username, server, self)
		self.vnc.connect("vnc-disconnected", self.vnc_disconnected, self.dialog, self)

	def vnc_initialized (src, vnc, username, server, self):
		print _("Connection initialized")
		title = "%s@%s - eagleeye" % (username, server)
		self.current_server = server
		self.dialog.hide()
		self.window.set_title(title)
		self.window_label.set_markup ("<big><b>%s@%s</b></big>" % (username, server))
		self.window.show_all()
		self.window_toolbar_note.hide_all()
		self.window.resize (vnc.get_width(), vnc.get_height())
		vnc.grab_focus()
		
	def vnc_disconnected(src, vnc, window, self):
		print _("Disconnected")
		dialog = gtk.MessageDialog (window,
			gtk.MESSAGE_INFO,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.BUTTONS_OK,
			_("You have been disconnected"))
		dialog.run()
		dialog.destroy()
		self.vnc.destroy()
		self.vnc=gtkvnc.Display()
		window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.ARROW))

	def vnc_connected(src, data, self):
		print _("Connected")
		self.layout.add(self.vnc)
		self.vnc.realize()


if __name__ == "__main__":
	instance = GtkVncViewer()
	gtk.main()
